!This subroutine calculates:
!	1.- the correlation matrix between the sample points and the points to predict: PCORR1
!   2.- the values of lamda for the points to predict: PLAM
!	3.- mean prediction = MZP and variance prediction = VZP

!OUTPUT: MZP,VZP

SUBROUTINE gpredicgrid(P,N,npr,PDIST1,thet2,phi,ICORR, &
						K, KH, CORR, omega2,gamma1,X,XP,bet,Z,MZP,VZP)

	USE NUMERICAL_LIBRARIES

	IMPLICIT NONE
	
	INTEGER :: I, J, P, T ,npr, k,N,KH,H
	DOUBLE PRECISION ::	PDIST1(npr,p), PCORR1(P),CORR(N,P),ICORR(N,P)
	DOUBLE PRECISION :: matern, thet2, phi,dt,Z(P)
	DOUBLE PRECISION :: X(N,KH), XP(npr,KH), bet(k), omega2, gamma1, PI2
	DOUBLE PRECISION :: A(P),A3,A2(P,P),IA2(P,P)
	DOUBLE PRECISION :: AX(K),VZP(npr),MZP(npr), AZ

!************************************************************************

PI2=1/dsqrt(2.0d0*3.14d0)

	!C_oo+w2*I_o=A2

	DO I=1,P-1
		A2(I,I)=omega2+1
		DO J=I+1,P
			A2(I,J)=CORR(I,J)
			A2(J,I)=A2(I,J)
		END DO
	END DO
	A2(P,P)=omega2+1

	!A2^{-1}=IA2
	CALL DLINDS(P,A2,P,IA2,P)


DO I=1,npr

!1.-PCORR1: Correlation vector for prediction: C_po
	
	DO J=1,P
		dt=phi*PDIST1(I,J)
			IF(dt.gt.7.05D+02) THEN
				PCORR1(J)=0.0d0
			ELSE
				PCORR1(J)= matern(thet2, phi, PDIST1(I,J))
			ENDIF
	END DO
		
!************************************************************************
!2.-

!*2.1  A=C_po*(C_oo+w2*I_n)^-1=C_po*IA2
	
	DO J=1,P
		A(J)=0
		DO H=1,P
			A(J)=A(J)+PCORR1(H)*IA2(H,J)
		END DO	
	END DO

	
!*2.2  (X_p-A*X_o)*beta+A*Z_o=MZP
	
	!A*X_o
	DO J=1,K	
		AX(J)=0
		DO H=1,P
			AX(J)=AX(J)+A(H)*X(H,J)
		END DO	
	END DO

	!A*Z_o=AZ
	AZ=0
	DO J=1,P
		AZ=AZ+A(J)*Z(J)
	END DO

	!(X_p-A*X_o)*beta=MZP
	
	MZP(I)=0
	DO J=1,K
		MZP(I)=MZP(I)+(XP(I,J)-AX(J))*BET(J)
	END DO
	
	!MZP+AZ_o=MZP
		
	MZP(I)=MZP(I)+AZ

!*2.3  VZP
	!A*PCORR1^t=A3
	
	A3=0
	DO J=1,P
		A3=A3+A(J)*PCORR1(J)
	END DO

	!VZP=sigma^2*(C_pp+w2*I_p-A3) values of the diagonal

	VZP(I)=(1+omega2-A3)/gamma1

END DO

END SUBROUTINE gpredicgrid





