!*******************************
!This subroutine calculates:
! 1.-Correlation matrix 
! 2.-Cholesky decomposition and the inverse of the upper triangular matrix 
! 3.-Inverse correlation matrix 
! 4.-Determinant of the correlation matrix
!*******************************

!INPUT: P, DIST, thet2, phi
!OUTPUT: CORR, ICORR, deter

SUBROUTINE CORRELATION6(P,N,DIST,thet2,phi,CORR,ICORR,deter)

	USE NUMERICAL_LIBRARIES

	IMPLICIT NONE
	
	INTEGER :: I, J, P, T ,N
	DOUBLE PRECISION ::	DIST(N,N),CORR(N,N),ICORR(N,N),IRSIG(P,P), RSIG(P,P)
	DOUBLE PRECISION :: matern, thet2, phi,dt, deter
	INTEGER :: LDRSIG, IRANK

!CORR: Correlation matrix
!Matern class

			DO I=1,P
				CORR(I,I)=1.0d0
				DO J=I+1,P
					dt=phi*DIST(I,J)
					IF(dt.gt.7.05D+02) THEN
						CORR(I,J)=0.0d0
						ELSE
						CORR(I,J)= matern(thet2, phi, DIST(I,J))
					ENDIF
					CORR(J,I)=CORR(I,J)
				END DO
			END DO


!Cholesky factorization. RSIG is the upper triangular matrix. t(RSIG)*RSIG=CORR
!RSIG=L


	LDRSIG=P 
	IRANK=P	  
	
	CALL DCHFAC(P, CORR, N, 1.00D-100, IRANK, RSIG, LDRSIG)

!Inversion of RSIG=IRSIG
	CALL DLINRT(P, RSIG, P, 2,IRSIG, P) !L^{-1} 

!Inverse matrix ICORR
	CALL DMXYTF(P,P,IRSIG,P,P,P,IRSIG,P,P,P,ICORR,N)

!Determinant of CORR
	deter=1
	DO I=1,P
		deter=deter*RSIG(I,I)**2
	END DO

END SUBROUTINE CORRELATION6

