!Sampling EPSILONS
!INPUT:P,N,CORR,LAM,OMEGA2,RES,gamma1
!OUTPUT:EPSI
	
	SUBROUTINE EPSILONS(P,N,CORR,LAM,OMEGA2,RES,gamma1,EPSI)

	IMPLICIT NONE

	INTEGER :: I, J, P, N
	INTEGER:: LDRSIG, IRANK	
	DOUBLE PRECISION :: CORR(N,N),  OMEGA2, LAM(P), RES(P), gamma1
	DOUBLE PRECISION :: CORR2(P,P), ICORR2(P,P), EPCORR(P,P), PSIG(P,P)
	DOUBLE PRECISION :: LWRES(P),EPSI(P),EPBET(P),EPNOR(P)

!1.-w^2*Lambda+C=CORR2

	DO I=1,P-1
		CORR2(I,I)=OMEGA2*LAM(I)+CORR(I,I)
		DO J=I+1,P
			CORR2(I,J)=CORR(J,I)
			CORR2(J,I)=CORR(J,I)
		END DO
	END DO			
	CORR2(P,P)=OMEGA2*LAM(P)+CORR(P,P)

!2.- (w^2*Lambda+C)^-1=ICORR2

	CALL DLINRG(P, CORR2, P, ICORR2, P)

!3.- (w^2*Lambda+C)^-1*C=ICORR2*CORR=CORR2
!NOTE: using the same space of CORR2 to store the new one. 	  

	CALL DMRRRR(P,P,ICORR2,P,P,P,CORR,N,P,P,CORR2,P) 

!4.- I-(w^2*Lambda+C)^-1*C=ICORR2

	ICORR2(P,P)=1-CORR2(P,P)
	DO I=1,P-1
		ICORR2(I,I)=1-CORR2(I,I)
		DO J=I+1,P
			ICORR2(I,J)=-CORR2(I,J)
			ICORR2(J,I)=-CORR2(J,I)
		END DO
	END DO

!5.- C*(I-(w^2*Lambda+C)^-1*C)=C*ICORR2=EPCORR   CORRELATION MATRIX FOR EPSILON

	CALL DMRRRR(P,P,CORR,N,P,P,ICORR2,P,P,P,EPCORR,P)  

!6.-CHOLESKY DECOMPOSITION OF EPCORR

	LDRSIG=P !The same as P
	IRANK=P	 !The same as P !1.00D-04
	
	CALL DCHFAC(P, EPCORR, P, 1.00D-100, IRANK, PSIG, LDRSIG)

!7.-EPBET: mu*=EPCORR*(LAM^{-1/2}/omega2)*(z-XB)

	DO I=1,P
		LWRES(I)=RES(I)*dsqrt(gamma1)/(dsqrt(LAM(I))*OMEGA2)
	END DO


	CALL DMURRV(P,P,EPCORR,P,P,LWRES,1,P,EPBET)


!NORMAL_n(EPBET,EPCORR)

	CALL DRNMVN(1,P,PSIG,P,EPNOR,1)


		DO J=1,P
			EPSI(J)=EPBET(J)+EPNOR(J)
		END DO

	END SUBROUTINE EPSILONS