!*******************************
!This subroutine calculates:
! 1.-Correlation matrix 
! 2.-Cholesky decomposition and the inverse of the upper triangular matrix 
! 3.-Inverse correlation matrix 
! 4.-Determinant of the correlation matrix
!*******************************

!INPUT: P,N,LAM,CORR,omega2
!OUTPUT: Mdeter,MICORR

SUBROUTINE MARGCORR(P,N,LAM,CORR,omega2,Mdeter,MICORR)

	USE NUMERICAL_LIBRARIES

	IMPLICIT NONE
	
	INTEGER :: I, J, P, T ,N
	DOUBLE PRECISION ::	CORR(N,N),IRSIG(P,P), RSIG(P,P),LAM(P),omega2
	INTEGER :: LDRSIG, IRANK
	DOUBLE PRECISION :: MCORR(P,P), MICORR(N,N), Mdeter

!MCORR: Correlation matrix=LAM^{-1/2}*CORR*LAM^{-1/2}+omega2*I
!Matern class	
	
	DO I=1,P-1
		MCORR(I,I)=(1/LAM(I))+omega2
		DO J=I+1,P
			MCORR(I,J)=CORR(I,J)/dsqrt(LAM(I)*LAM(J))
			MCORR(J,I)=MCORR(I,J)
		END DO
	END DO
	MCORR(P,P)=(1/LAM(P))+omega2

!Cholesky factorization. RSIG is the upper triangular matrix. t(RSIG)*RSIG=CORR
!RSIG=L

	LDRSIG=P 
	IRANK=P	 
	
	CALL DCHFAC(P, MCORR, P, 1.00D-100, IRANK, RSIG, LDRSIG)

!Inversion of RSIG=IRSIG
	CALL DLINRT(P, RSIG, P, 2,IRSIG, P) !L^{-1} 


!Inverse matrix MICORR
	CALL DMXYTF(P,P,IRSIG,P,P,P,IRSIG,P,P,P,MICORR,N)

! Determinant of MCORR
	Mdeter=1
	DO I=1,P
		Mdeter=Mdeter*RSIG(I,I)**2
	END DO

END SUBROUTINE MARGCORR