!This subroutine calculates:
!	1.- the correlation matrix between the sample points and the points to predict: PCORR1
!   2.- the values of lambda for the points to predict: PLAM
!	3.- mean prediction = MZP and variance prediction = VZP

!OUTPUT: MZP,VZP

SUBROUTINE predicgrid(P,N,npr,PDIST1,thet2,phi,ICORR,LAM,lnu, &
						K, KH, CORR, omega2,gamma1, X,XP,bet,Z, MZP,VZP)

	USE NUMERICAL_LIBRARIES

	IMPLICIT NONE
	
	INTEGER :: I, J, P, T ,npr, k,N,KH, H
	DOUBLE PRECISION ::	PDIST1(npr,P), PCORR1(P),CORR(N,P),ICORR(N,P)
	DOUBLE PRECISION :: matern, thet2, phi,dt,Z(P), LAM(P),lnu
	DOUBLE PRECISION :: CPO(P), CPO1, CLAM, CPO2
	DOUBLE PRECISION :: VPLAM(npr),MPLAM(npr),PLAM(npr),PLAM1
	DOUBLE PRECISION :: X(N,KH), XP(npr,KH), bet(k), omega2, gamma1, PI2
	DOUBLE PRECISION :: A(P), A1(P),A2(P,P),IA2(P,P),A3
	DOUBLE PRECISION :: AX(K),VZP(npr),MZP(npr), AZ

!************************************************************************

PI2=1/dsqrt(2.0d0*3.14d0)

	!LAM_o^{-1/2}*C_oo*LAM_o^{-1/2}+w2*I_o=A2

	DO I=1,P-1
		A2(I,I)=omega2+1/LAM(I)
		DO J=I+1,P
			A2(I,J)=CORR(I,J)/dsqrt(LAM(I)*LAM(J))
			A2(J,I)=A2(I,J)
		END DO
	END DO
	A2(P,P)=omega2+1/LAM(P)

	!A2^{-1}=IA2
	CALL DLINDS(P,A2,P,IA2,P)

DO I=1,npr

	!1.-PCORR1: Correlation vector for prediction: C_po
	
	DO J=1,P
		dt=phi*PDIST1(I,J)
			IF(dt.gt.7.05D+02) THEN
				PCORR1(J)=0.0d0
			ELSE
				PCORR1(J)= matern(thet2, phi, PDIST1(I,J))
			ENDIF
	END DO
		

!************************************************************************

!2.-LAM_p

	!C_po*C_oo^{-1}=CPO
	!C_po*C_oo^{-1}*1=CPO1
	!C_po*C_oo^{-1}*ln(LAMBDA_o)=CLAM
	!C_po*C_oo^{-1}*C_op=CPO2
	


	CPO1=0
	CLAM=0
	CPO2=0

	DO J=1,P
		CPO(J)=0
		DO H=1,P
			CPO(J)=CPO(J)+PCORR1(H)*ICORR(H,J)
		END DO
		CPO1=CPO1+CPO(J)
		CLAM=CLAM+CPO(J)*DLOG(LAM(J))
		CPO2=CPO2+CPO(J)*PCORR1(J)
	END DO	
	
	!MEAN log(LAM_p)

	Mplam(I)=lnu*0.5d0*(CPO1-1.0D0)+CLAM



	!Var(log(LAM_p))

	VPLAM(I)=lnu*(1.0d0-CPO2)
	IF(VPLAM(I).LT.0.0d0) THEN 
		PRINT *, 'The variance of lambda is zero.',I, VPLAM(I)
		VPLAM(I)=0.0d0
	END IF

	!log LAM_p| log LAM_o  Normal(Mplam,VPLAM)

	CALL DRNNOR(1,PLAM1)
	
	PLAM(I)=dexp(Mplam(I)+PLAM1*VPLAM(I))

!************************************************************************

!3.-A
	!LAM_p^{-1/2}*C_po*LAM_o^{-1/2}=A1


	DO J=1,P
		A1(J)=PCORR1(J)/DSQRT(LAM(J)*PLAM(I))
	END DO


	DO J=1,P
		A(J)=0
		DO H=1,P
			A(J)=A(J)+A1(H)*IA2(H,J)
		END DO	
	END DO
	
! (X_p-A*X_o)*beta+A*Z_o=MZP
	
	!A*X_o
	DO J=1,K	
		AX(J)=0
		DO H=1,P
			AX(J)=AX(J)+A(H)*X(H,J)
		END DO	
	END DO

	!A*Z_o=AZ
	AZ=0
	DO J=1,P
		AZ=AZ+A(J)*Z(J)
	END DO

	!(X_p-A*X_o)*beta=MZP
	
	MZP(I)=0
	DO J=1,K
		MZP(I)=MZP(I)+(XP(I,J)-AX(J))*BET(J)
	END DO

	!MZP+A*Z_o=MZP
	MZP(I)=MZP(I)+AZ


!VZP
	!A*A1^t=A3

	A3=0
	DO J=1,P
		A3=A3+A(J)*A1(J)
	END DO


	!LAM_p^{-1/2}*C_pp*LAM_p^{-1/2}+w2*I_p=LCL1	values of the diagonal

	!VZP=(LCL1-A3)*sigma^2 values of the diagonal

	VZP(I)=(omega2+1/PLAM(I)-A3)/gamma1

END DO


END SUBROUTINE predicgrid



!*****************************************************

